"""
Locations

Locations are used to organize Webex Calling (BroadCloud) features within physical locations. Webex Control Hub may be
used to define new locations.

Searching and viewing locations in your organization requires an administrator auth token with the
spark-admin:people_read and spark-admin:people_write or spark-admin:device_read AND spark-admin:device_writescope
combinations.
"""

from collections.abc import Generator
from typing import Optional

from pydantic import Field

from .api_child import ApiChild
from .base import ApiModel, to_camel, webex_id_to_uuid

__all__ = ['LocationAddress', 'Location', 'LocationsApi']


class LocationAddress(ApiModel):
    """
    Address of a :class:`Location`
    """

    #: address line 1
    address1: str
    #: address line 2
    address2: str
    #: city
    city: str
    #: state
    state: Optional[str]
    #: ZIP/Postal code
    postal_code: str
    #: country
    country: str


class Location(ApiModel):
    """
    Webex location

    """
    #: A unique identifier for the location.
    location_id: str = Field(alias='id')
    #: The name of the location.
    name: str
    #: The ID of the organization to which this location belongs.
    org_id: str
    #: The address of the location, :class:`LocationAddress`
    address: LocationAddress

    @property
    def location_id_uuid(self) -> str:
        """
        location id as UUID
        """
        return webex_id_to_uuid(self.location_id)

    @property
    def org_id_uuid(self) -> str:
        """
        org id as UUID
        """
        return webex_id_to_uuid(self.org_id)


class LocationsApi(ApiChild, base='locations'):
    """
    Location API

    Locations are used to organize Webex Calling (BroadCloud) features within physical locations. Webex Control Hub
    may be used to define new locations.

    Searching and viewing locations in your organization requires an administrator auth token with the
    spark-admin:people_read and spark-admin:people_write or spark-admin:device_read AND spark-admin:device_write
    scope combinations.
    """

    def list(self, name: str = None, location_id: str = None, org_id: str = None) -> Generator[Location, None, None]:
        """
        List locations for an organization.

        :param name: List locations whose name contains this string (case-insensitive).
        :type name: str
        :param location_id: List locations by ID.
        :type location_id: str
        :param org_id: List locations in this organization. Only admin users of another organization
            (such as partners) may use this parameter.
        :type org_id: str
        :return: generator of :class:`Location` instances
        """
        data = {to_camel(k): v for i, (k, v) in enumerate(locals().items())
                if i and k != 'self' and v is not None}
        if location_id is not None:
            data.pop('locationId')
            data['id'] = location_id
        ep = self.ep()
        # noinspection PyTypeChecker
        return self.session.follow_pagination(url=ep, model=Location, params=data)

    def by_name(self, name: str, org_id: str = None) -> Optional[Location]:
        """
        Get a location by name

        :param name: name of the location to search
        :type name: str
        :param org_id: search in list of locations  in this organization. Only admin users of another organization
            (such as partners) may use this parameter.
        :type org_id: str
        :return: locations
        :rtype: Location
        """
        return next((location for location in self.list(name=name, org_id=org_id)
                     if location.name == name), None)

    def details(self, location_id) -> Location:
        """
        Shows details for a location, by ID.

        This API only works for Customer administrators and for Partner administrators to query their own organization.
        Partner administrators looking to query customer organizations should use the List Locations endpoint to
        retrieve information about locations.

        :param location_id: A unique identifier for the location.
        :type location_id: str
        :return: location details
        :rtype: Location
        """
        ep = self.ep(location_id)
        return Location.parse_obj(self.get(ep))



"""
Person barge settings API
"""

from .common import PersonSettingsApiChild
from ..base import ApiModel

__all__ = ['BargeSettings', 'BargeApi']


class BargeSettings(ApiModel):
    """
    Barge settings
    """
    #: indicates if the Barge In feature is enabled.
    enabled: bool
    #: Indicates that a stutter dial tone will be played when a person is barging in on the active call.
    tone_enabled: bool


class BargeApi(PersonSettingsApiChild, base='people'):
    """
    Api for person's barge settings
    """

    def read(self, person_id: str, org_id: str = None) -> BargeSettings:
        """
        Retrieve a Person's Barge In Settings

        The Barge In feature enables you to use a Feature Access Code (FAC) to answer a call that was directed to
        another subscriber, or barge-in on the call if it was already answered. Barge In can be used across locations.

        This API requires a full, user, or read-only administrator auth token with a scope of spark-admin:people_read
        or a user auth token with spark:people_read scope can be used by a person to read their own settings.

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param org_id: Person is in this organization. Only admin users of another organization (such as partners)
            may use this parameter as the default is the same organization as the token used to access API.
        :type org_id: str
        :return: barge settings for specific user
        :rtype: BargeSettings
        """
        ep = self.f_ep(person_id=person_id, path='bargeIn')
        params = org_id and {'orgId': org_id} or None
        return BargeSettings.parse_obj(self.get(ep, params=params))

    def configure(self, person_id: str, barge_settings: BargeSettings, org_id: str = None):
        """
        Configure a Person's Barge In Settings

        The Barge In feature enables you to use a Feature Access Code (FAC) to answer a call that was directed to
        another subscriber, or barge-in on the call if it was already answered. Barge In can be used across locations.

        This API requires a full or user administrator auth token with the spark-admin:people_write scope or a user
        auth token with spark:people_write scope can be used by a person to update their own settings.

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param barge_settings: new setting to be applied
        :type barge_settings: BargeSettings
        :param org_id: Person is in this organization. Only admin users of another organization (such as partners)
            may use this parameter as the default is the same organization as the token used to access API.
        """
        ep = self.f_ep(person_id=person_id, path='bargeIn')
        params = org_id and {'orgId': org_id} or None
        self.put(ep, params=params, data=barge_settings.json())



"""
Call queue announcement files API
"""

from collections.abc import Generator

from pydantic import Field

from ...base import ApiModel
from ...rest import RestSession

__all__ = ['AnnouncementApi', 'Announcement']


class Announcement(ApiModel):
    """
    Announcement file information
    """
    name: str = Field(alias='fileName')
    size: int = Field(alias='fileSize')


class AnnouncementApi:
    """
    API for call queue Announcements
    """

    def __init__(self, *, session: RestSession):
        self._session = session

    def _endpoint(self, location_id: str, queue_id: str, path: str = None):
        """

        :meta private:
        :param location_id:
        :param queue_id:
        :param path:
        :return:
        """
        path = path and f'/{path}' or ''
        ep = self._session.ep(path=f'telephony/config/locations/{location_id}/queues/{queue_id}/announcements{path}')
        return ep

    def list(self, *, location_id: str, queue_id: str, org_id: str = None) -> Generator[Announcement]:
        """

        :param location_id:
        :param queue_id:
        :param org_id:
        :return:
        """
        url = self._endpoint(location_id=location_id, queue_id=queue_id)
        params = org_id and {'orgId': org_id} or dict()
        # noinspection PyTypeChecker
        return self._session.follow_pagination(url=url, model=Announcement, params=params)

    def delete_announcement(self, *, location_id: str, queue_id: str, file_name: str, org_id: str = None):
        """

        :param location_id:
        :type location_id: str
        :param queue_id:
        :type queue_id: str
        :param file_name:
        :type file_name: str
        :param org_id:
        """
        url = self._endpoint(location_id=location_id, queue_id=queue_id, path=file_name)
        params = org_id and {'orgId': org_id} or None
        self._session.delete(url=url, params=params)



from .rest import RestSession, StrOrDict

__all__ = ['ApiChild']


class ApiChild:
    """
    Base class for child APIs of :class:`WebexSimpleApi`
    """

    def __init__(self, session: RestSession):
        #: REST session
        self.session = session

    def __init_subclass__(cls, base: str, **kwargs):
        """
        Subclass registration hook. Each APIChild has a specific endpoint prefix which we gather at subclass
        registration time-

        :param base: APIChild specific URL path
        :param kwargs:
        """
        super().__init_subclass__(**kwargs)
        # save endpoint prefix
        cls.base = base

    def ep(self, path: str = None):
        """
        endpoint URL for given path

        :param path: path after APIChild subclass specific endpoint URI prefix
        :type path: str
        :return: endpoint URL
        :rtype: str
        """
        path = path and f'/{path}' or ''
        return self.session.ep(f'{self.base}{path}')

    def get(self, *args, **kwargs) -> StrOrDict:
        """
        GET request

        :param args:
        :param kwargs:
        :return:
        """
        return self.session.rest_get(*args, **kwargs)

    def post(self, *args, **kwargs) -> StrOrDict:
        """
        POST request

        :param args:
        :param kwargs:
        :return:
        """
        return self.session.rest_post(*args, **kwargs)

    def put(self, *args, **kwargs) -> StrOrDict:
        """
        PUT request

        :param args:
        :param kwargs:
        :return:
        """
        return self.session.rest_put(*args, **kwargs)

    def delete(self, *args, **kwargs) -> None:
        """
        DELETE request

        :param args:
        :param kwargs:
        """
        self.session.rest_delete(*args, **kwargs)



"""
Simple SDK for Webex APIs with focus on Webex Calling specific endpoints
"""
import logging
import os
from typing import Union

from .licenses import LicensesApi
from .locations import LocationsApi
from .people import PeopleApi
from .person_settings import PersonSettingsApi
from .rest import RestSession
from .telephony import TelephonyApi
from .tokens import Tokens
from .webhook import WebhookApi
from dataclasses import dataclass

__all__ = ['WebexSimpleApi']

__version__ = '0.4.0'

log = logging.getLogger(__name__)


@dataclass(init=False)
class WebexSimpleApi:
    """
    The main API object

    """

    #: Licenses API :class:`licenses.LicensesApi`
    licenses: LicensesApi
    #: Location API :class:`locations.LocationsApi`
    locations: LocationsApi
    #: Person settings API :class:`person_settings.PersonSettingsApi`
    person_settings: PersonSettingsApi
    #: People API :class:`people.PeopleApi`
    people: PeopleApi
    #: Telephony API :class:`telephony.TelephonyApi`
    telephony: TelephonyApi
    #: Webhooks API :class:`webhook.WebhookApi`
    webhook: WebhookApi
    #: :class:`rest.RestSession` used for all API requests
    session: RestSession

    def __init__(self, *, tokens: Union[str, Tokens] = None, concurrent_requests: int = 10):
        """

        :param tokens: token to be used by the API. Can be a :class:`tokens.Tokens` instance, a string or None. If
            None then an access token is expected in the WEBEX_ACCESS_TOKEN environment variable.
        :param concurrent_requests: number of concurrent requests when using multi-threading
        :type concurrent_requests: int
        """
        if isinstance(tokens, str):
            tokens = Tokens(access_token=tokens)
        elif tokens is None:
            tokens = os.getenv('WEBEX_ACCESS_TOKEN')
            if tokens is None:
                raise ValueError('if no access token is passed, then a valid access token has to be present in '
                                 'WEBEX_ACCESS_TOKEN environment variable')
            tokens = Tokens(access_token=tokens)

        session = RestSession(tokens=tokens, concurrent_requests=concurrent_requests)
        self.licenses = LicensesApi(session=session)
        self.locations = LocationsApi(session=session)
        self.person_settings = PersonSettingsApi(session=session)
        self.people = PeopleApi(session=session)
        self.telephony = TelephonyApi(session=session)
        self.webhook = WebhookApi(session=session)
        self.session = session

    def close(self):
        self.session.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



"""
Call intercept API
"""
import json
import os
from enum import Enum
from io import BufferedReader
from typing import Optional, Union

from pydantic import Field
from requests_toolbelt.multipart.encoder import MultipartEncoder
from .common import PersonSettingsApiChild
from ..base import ApiModel
from ..common import Greeting


class InterceptTypeIncoming(str, Enum):
    #: incoming calls are intercepted. Incoming calls are routed as destination and voicemail specify.
    intercept_all = 'INTERCEPT_ALL'
    #: Incoming calls are not intercepted.
    allow_all = 'ALLOW_ALL'


class InterceptNumber(ApiModel):
    """
    Information about a number announcement.
    """
    #: If true, the caller will hear this number when the call is intercepted.
    enabled: bool
    #: number caller will hear announced.
    destination: Optional[str]


class InterceptAnnouncements(ApiModel):
    """
    Settings related to how incoming calls are handled when the intercept feature is enabled.
    """
    greeting: Greeting
    #: Filename of custom greeting, will be an empty string if no custom greeting has been uploaded.
    file_name: Optional[str]
    #: Information about the new number announcement.
    new_number: InterceptNumber
    #: Information about how the call will be handled if zero (0) is pressed.
    zero_transfer: InterceptNumber

    @staticmethod
    def default() -> 'InterceptAnnouncements':
        return InterceptAnnouncements(greeting=Greeting.default, new_number=InterceptNumber(enabled=False),
                                      zero_transfer=InterceptNumber(enabled=False))


class InterceptSettingIncoming(ApiModel):
    """
    Settings related to how incoming calls are handled when the intercept feature is enabled.
    """
    intercept_type: InterceptTypeIncoming = Field(alias='type')
    #: If true, the destination will be the person's voicemail.
    voicemail_enabled: bool
    #: Settings related to how incoming calls are handled when the intercept feature is enabled.
    announcements: InterceptAnnouncements

    @staticmethod
    def default() -> 'InterceptSettingIncoming':
        return InterceptSettingIncoming(intercept_type=InterceptTypeIncoming.intercept_all, voicemail_enabled=False,
                                        announcements=InterceptAnnouncements.default())


class InterceptTypeOutgoing(str, Enum):
    #: Outgoing calls are routed as destination and voicemail specify.
    intercept_all = 'INTERCEPT_ALL'
    #: Only non-local calls are intercepted.
    allow_local_only = 'ALLOW_LOCAL_ONLY'


class InterceptSettingOutgoing(ApiModel):
    intercept_type: InterceptTypeOutgoing = Field(alias='type')
    #: If true, when the person attempts to make an outbound call, a system default message is played and the call is
    #: made to the destination phone number
    transfer_enabled: bool
    #: Number to which the outbound call be transferred.
    destination: Optional[str]

    @staticmethod
    def default() -> 'InterceptSettingOutgoing':
        return InterceptSettingOutgoing(intercept_type=InterceptTypeOutgoing.intercept_all, transfer_enabled=False)


class InterceptSetting(ApiModel):
    """
    A person's call intercept settings
    """
    #: true if call intercept is enabled.
    enabled: bool
    #: Settings related to how incoming calls are handled when the intercept feature is enabled.
    incoming: InterceptSettingIncoming
    #: Settings related to how outgoing calls are handled when the intercept feature is enabled.
    outgoing: InterceptSettingOutgoing

    @staticmethod
    def default() -> 'InterceptSetting':
        return InterceptSetting(enabled=False,
                                incoming=InterceptSettingIncoming.default(),
                                outgoing=InterceptSettingOutgoing.default())


class CallInterceptApi(PersonSettingsApiChild, base='people'):
    """
    Api for person's call intercept settings
    """

    def read(self, person_id: str, org_id: str = None) -> InterceptSetting:
        """
        Read Call Intercept Settings for a Person

        Retrieves Person's Call Intercept Settings

        The intercept feature gracefully takes a person’s phone out of service, while providing callers with
        informative announcements and alternative routing options. Depending on the service configuration, none,
        some, or all incoming calls to the specified person are intercepted. Also depending on the service
        configuration, outgoing calls are intercepted or rerouted to another location.

        This API requires a full, user, or read-only administrator auth token with a scope of spark-admin:people_read.

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param org_id: Person is in this organization. Only admin users of another organization (such as partners)
            may use this parameter as the default is the same organization as the token used to access API.
        :type org_id: str
        :return: user's call intercept settings
        :rtype: InterceptSetting
        """
        ep = self.f_ep(person_id=person_id, path='intercept')
        params = org_id and {'orgId': org_id} or None
        return InterceptSetting.parse_obj(self.get(ep, params=params))

    def configure(self, person_id: str, intercept: InterceptSetting, org_id: str = None):
        """
        Configure Call Intercept Settings for a Person
        Configures a Person's Call Intercept Settings

        The intercept feature gracefully takes a person’s phone out of service, while providing callers with
        informative announcements and alternative routing options. Depending on the service configuration, none, some,
        or all incoming calls to the specified person are intercepted. Also depending on the service configuration,
        outgoing calls are intercepted or rerouted to another location.

        This API requires a full or user administrator auth token with the spark-admin:people_write scope.

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param intercept: new intercept settings
        :type intercept: InterceptSetting
        :param org_id: Person is in this organization. Only admin users of another organization (such as partners)
            may use this parameter as the default is the same organization as the token used to access API.
        :type org_id: str
        """
        ep = self.f_ep(person_id=person_id, path='intercept')
        params = org_id and {'orgId': org_id} or None
        data = json.loads(intercept.json())
        try:
            # remove attribute not present in update
            data['incoming']['announcements'].pop('fileName', None)
        except KeyError:
            pass
        self.put(ep, params=params, json=data)

    def greeting(self, person_id: str, content: Union[BufferedReader, str],
                 upload_as: str = None, org_id: str = None):
        """
        Configure Call Intercept Greeting for a Person

        Configure a Person's Call Intercept Greeting by uploading a Waveform Audio File Format, .wav, encoded audio
        file.

        Your request will need to be a multipart/form-data request rather than JSON, using the audio/wav Content-Type.

        This API requires a full or user administrator auth token with the spark-admin:people_write scope or a user
        auth token with spark:people_write scope can be used by a person to update their settings.

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param content: the file to be uploaded, can be a path to a file or a buffered reader (opened file); if a
            reader referring to an open file is passed then make sure to open the file as binary b/c otherwise the
            content length might be calculated wrong
        :type content: Union[BufferedReader, str]
        :param upload_as: filename for the content. Only required if content is a reader; has to be a .wav file name.
        :type upload_as: str
        :param org_id: Person is in this organization. Only admin users of another organization (such as partners)
            may use this parameter as the default is the same organization as the token used to access API.
        :type org_id: str
        """
        if isinstance(content, str):
            upload_as = os.path.basename(content)
            content = open(content, mode='rb')
            must_close = True
            pass
        else:
            must_close = False
            # an existing reader
            if not upload_as:
                raise ValueError('upload_as is required')
        encoder = MultipartEncoder(fields={'file': (upload_as, content, 'audio/wav')})
        ep = self.f_ep(person_id=person_id, path='intercept/actions/announcementUpload/invoke')
        params = org_id and {'orgId': org_id} or None
        try:
            self.post(ep, data=encoder, headers={'Content-Type': encoder.content_type},
                      params=params)
        finally:
            if must_close:
                content.close()
        return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wxc_sdk',
 'wxc_sdk.person_settings',
 'wxc_sdk.telephony',
 'wxc_sdk.telephony.callqueue']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=1.11.1,<2.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'pytz',
 'requests-toolbelt>=0.9.1,<0.10.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'wxc-sdk',
    'version': '0.4.1',
    'description': 'SDK for Webex APIs with special focus on Webex Calling specific endpoints',
    'long_description': "=======\nwxc_sdk\n=======\n\nA simple SDK to work with `Webex APIs <https://developer.webex.com>`_, special focus on Webex Calling specific API\nendpoints.\n\n----------------------------------------------\n\nThis is how easy it is to use the SDK. The example code list all calling enabled users within the org.\n\n.. code-block:: Python\n\n    from wxc_sdk import WebexSimpleApi\n\n    api = WebexSimpleApi()\n\n    # if a user is enabled for calling, then the location_id attribute is set\n    calling_users = [user for user in api.people.list(calling_data=True)\n                     if user.location_id]\n    print(f'{len(calling_users)} users:')\n    print('\\n'.join(user.display_name for user in calling_users))\n\n\nInstallation\n------------\n\nInstalling and upgrading wxc_sdk is easy:\n\n**Install via PIP**\n\n.. code-block:: bash\n\n    $ pip install wxc-sdk\n\n**Upgrade to the latest version**\n\n.. code-block:: bash\n\n    $ pip install wxc-sdk --upgrade\n\nDocumentation\n-------------\n\nDocumentation is available at:\nhttp://wxc_sdk.readthedocs.io\n\nExamples\n--------\n\nSample scripts are available in the examples_ folder.\n\n.. _examples: https://github.com/jeokrohn/wxc_sdk/tree/master/examples\n",
    'author': 'Johannes Krohn',
    'author_email': 'jkrohn@cisco.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jeokrohn/wxc_sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from ..api_child import ApiChild

__all__ = ['PersonSettingsApiChild']


class PersonSettingsApiChild(ApiChild, base='people'):
    """
    API for all user level settings
    """

    def f_ep(self, *, person_id: str, path: str = None) -> str:
        """
        person specific feature endpoint like v1/people/{uid}/features/....

        :param person_id: Unique identifier for the person.
        :type person_id: str
        :param path: path in the endpoint after the feature base URL
        :type path: str
        :return: full endpoint
        :rtype: str
        """
        path = path and f'/{path}' or ''
        return self.session.ep(f'people/{person_id}/features{path}')



from wxc_sdk.common import *
from wxc_sdk.licenses import *
from wxc_sdk.locations import *
from wxc_sdk.people import *
from wxc_sdk.person_settings import *
from wxc_sdk.person_settings.barge import *
from wxc_sdk.person_settings.call_recording import *
from wxc_sdk.person_settings.caller_id import *
from wxc_sdk.person_settings.common import *
from wxc_sdk.person_settings.dnd import *
from wxc_sdk.person_settings.forwarding import *
from wxc_sdk.person_settings.voicemail import *
from wxc_sdk.telephony import *
from wxc_sdk.telephony.autoattendant import *
from wxc_sdk.telephony.callqueue import *
from wxc_sdk.telephony.callqueue.announcement import *
from wxc_sdk.telephony.calls import *
from wxc_sdk.telephony.forwarding import *
from wxc_sdk.telephony.hg_and_cq import *
from wxc_sdk.telephony.huntgroup import *
from wxc_sdk.telephony.paging import *
from wxc_sdk.telephony.schedules import *
from wxc_sdk.telephony.user_base import *
from wxc_sdk.tokens import *
from wxc_sdk.webhook import *

__all__ = ['Agent', 'AlternateNumber', 'AlternateNumberSettings', 'Announcement', 'AudioSource', 'AutoAttendant', 
           'AutoAttendantAction', 'AutoAttendantKeyConfiguration', 'AutoAttendantMenu', 'BargeSettings', 'CallBounce', 
           'CallForwarding', 'CallForwardingAlways', 'CallForwardingCommon', 'CallForwardingNoAnswer', 
           'CallForwardingNumber', 'CallForwardingNumberType', 'CallForwardingPerson', 'CallHistoryRecord', 
           'CallPolicies', 'CallQueue', 'CallQueueDetail', 'CallRecordingSetting', 'CallState', 'CallType', 
           'CallerId', 'CallerIdSelectedType', 'CallsFrom', 'ComfortMessageSetting', 'CustomNumberInfo', 
           'CustomNumberType', 'CustomNumbers', 'DND', 'DialResponse', 'Dialing', 'DistinctiveRing', 'Event', 
           'ExternalCallerIdNamePolicy', 'FeatureSelector', 'ForwardCallsTo', 'ForwardToSelection', 'ForwardingRule', 
           'ForwardingRuleDetails', 'ForwardingSetting', 'Greeting', 'HGandCQ', 'HistoryType', 'HuntGroup', 
           'HuntGroupDetail', 'License', 'Location', 'LocationAddress', 'MenuKey', 'MohMessageSetting', 
           'Notification', 'NotificationRepeat', 'NotificationType', 'OverflowSetting', 'Paging', 'PeopleStatus', 
           'Person', 'PersonForwardingSetting', 'PersonSettingsApiChild', 'PersonType', 'Personality', 'PhoneNumber', 
           'PhoneNumberType', 'Policy', 'QueueSettings', 'Recall', 'Record', 'RecordingState', 'RecurWeekly', 
           'RecurYearlyByDate', 'RecurYearlyByDay', 'Recurrence', 'RedirectReason', 'Redirection', 'RingPattern', 
           'Schedule', 'ScheduleDay', 'ScheduleMonth', 'ScheduleType', 'ScheduleWeek', 'SipAddress', 'SipType', 
           'SiteType', 'StorageType', 'TelephonyCall', 'TelephonyEvent', 'TelephonyEventData', 'TelephonyParty', 
           'Tokens', 'UnansweredCalls', 'UserBase', 'UserType', 'VoiceMailFax', 'VoicemailCopyOfMessage', 
           'VoicemailEnabled', 'VoicemailEnabledWithGreeting', 'VoicemailFax', 'VoicemailMessageStorage', 
           'VoicemailNotifications', 'VoicemailSettings', 'VoicemailTransferToNumber', 'WaitMessageSetting', 
           'WebHook', 'WebHookCreate', 'WebHookEvent', 'WebHookResource', 'WebHookStatus', 'WelcomeMessageSetting', ]



from collections.abc import Generator
from enum import Enum
from typing import List, Optional

from pydantic import Field

from .announcement import AnnouncementApi
from ..forwarding import ForwardingApi, FeatureSelector
from ..hg_and_cq import HGandCQ, Policy, Agent, AlternateNumberSettings
from ...base import to_camel, ApiModel
from ...common import RingPattern
from ...rest import RestSession

__all__ = ['CallQueueApi', 'CallQueue', 'CallQueueDetail', 'CallBounce', 'DistinctiveRing',
           'CallPolicies', 'OverflowSetting', 'WaitMessageSetting', 'WelcomeMessageSetting', 'AudioSource',
           'ComfortMessageSetting', 'MohMessageSetting', 'QueueSettings']


class CallQueue(HGandCQ):
    """
    A Call Queue object
    """
    pass


class CallBounce(ApiModel):
    """
    Call bounce settings
    """
    enabled: bool = Field(alias='callBounceEnabled')
    max_rings: Optional[int] = Field(alias='callBounceMaxRings')
    agent_unavailable_enabled: bool
    alert_agent_enabled: Optional[bool]
    alert_agent_max_seconds: Optional[int]
    on_hold_enabled: bool = Field(alias='callBounceOnHoldEnabled')
    on_hold_max_seconds: Optional[int] = Field(alias='callBounceOnHoldMaxSeconds')

    @staticmethod
    def default() -> 'CallBounce':
        return CallBounce(enabled=True,
                          max_rings=8,
                          agent_unavailable_enabled=False,
                          alert_agent_enabled=False,
                          alert_agent_max_seconds=30,
                          on_hold_enabled=False,
                          on_hold_max_seconds=60)


class DistinctiveRing(ApiModel):
    """
    Distinctive ring settings
    """
    enabled: Optional[bool]
    ring_pattern: Optional[RingPattern]

    @staticmethod
    def default() -> 'DistinctiveRing':
        return DistinctiveRing(enabled=True,
                               ring_pattern=RingPattern.normal)


class CallPolicies(ApiModel):
    """
    Policy controlling how calls are routed to agents.
    """
    #: Call routing policy to use to dispatch calls to agents.
    policy: Optional[Policy]
    #: Settings for when the call into the call queue is not answered.
    call_bounce: Optional[CallBounce]
    #: Whether or not the call queue has the distinctive ring option enabled.
    distinctive_ring: Optional[DistinctiveRing]

    @staticmethod
    def default() -> 'CallPolicies':
        return CallPolicies(policy=Policy.circular,
                            call_bounce=CallBounce.default(),
                            distinctive_ring=DistinctiveRing.default())

    @staticmethod
    def simple() -> 'CallPolicies':
        return CallPolicies(policy=Policy.circular)


class OverflowAction(str, Enum):
    perform_busy_treatment = 'PERFORM_BUSY_TREATMENT'
    transfer_to_phone_number = 'TRANSFER_TO_PHONE_NUMBER'
    play_ringing_until_caller_hangs_up = 'PLAY_RINGING_UNTIL_CALLER_HANGS_UP'


class Greeting(str, Enum):
    default = 'DEFAULT'
    custom = 'CUSTOM'


class OverflowSetting(ApiModel):
    action: Optional[OverflowAction]
    send_to_voicemail: Optional[bool]
    transfer_number: Optional[str]
    is_transfer_number_set: Optional[bool]
    overflow_after_wait_enabled: Optional[bool]
    overflow_after_wait_time: Optional[int]
    play_overflow_greeting_enabled: Optional[bool]
    greeting: Optional[Greeting]
    audio_files: Optional[List[str]]

    @staticmethod
    def default() -> 'OverflowSetting':
        return OverflowSetting(action=OverflowAction.perform_busy_treatment,
                               send_to_voicemail=False,
                               is_transfer_number_set=False,
                               overflow_after_wait_enabled=False,
                               overflow_after_wait_time=30,
                               play_overflow_greeting_enabled=False,
                               greeting=Greeting.default,
                               audio_files=list())


class WaitMode(str, Enum):
    time = 'TIME'
    position = 'POSITION'


class WaitMessageSetting(ApiModel):
    enabled: Optional[bool]
    wait_mode: Optional[WaitMode]
    handling_time: Optional[int]
    queue_position: Optional[int]
    high_volume_message_enabled: Optional[bool]
    default_handling_time: Optional[int]

    @staticmethod
    def default():
        return WaitMessageSetting(enabled=False,
                                  wait_mode=WaitMode.position,
                                  handling_time=100,
                                  queue_position=100,
                                  high_volume_message_enabled=False,
                                  default_handling_time=5)


class AudioSource(ApiModel):
    enabled: bool = Field(default=True)
    greeting: Greeting = Field(default=Greeting.default)
    audio_files: List[str] = Field(default_factory=list)


class WelcomeMessageSetting(AudioSource):
    always_enabled: bool = Field(default=False)


class ComfortMessageSetting(AudioSource):
    time_between_messages: int = Field(default=10)

    @staticmethod
    def default() -> 'ComfortMessageSetting':
        return ComfortMessageSetting(enabled=False)


class MohMessageSetting(ApiModel):
    normal_source: AudioSource
    alternate_source: AudioSource

    @staticmethod
    def default() -> 'MohMessageSetting':
        return MohMessageSetting(normal_source=AudioSource(enabled=True),
                                 alternate_source=AudioSource(enabled=False))


class QueueSettings(ApiModel):
    queue_size: int
    call_offer_tone_enabled: bool = Field(default=True)
    reset_call_statistics_enabled: bool = Field(default=True)
    overflow: OverflowSetting = Field(default_factory=OverflowSetting)
    wait_message: WaitMessageSetting = Field(default_factory=WaitMessageSetting)
    welcome_message: WelcomeMessageSetting = Field(default_factory=WelcomeMessageSetting)
    comfort_message: ComfortMessageSetting = Field(default_factory=ComfortMessageSetting.default)
    moh_message: MohMessageSetting = Field(default_factory=MohMessageSetting.default)


class CallQueueDetail(HGandCQ):
    """
    Call queue details
    """
    #: Language for call queue.
    language: Optional[str]
    #: Language code.
    language_code: Optional[str]
    #: First name to be shown when calls are forwarded out of this call queue. Defaults to ".".
    first_name: Optional[str]
    #: Last name to be shown when calls are forwarded out of this call queue. Defaults to the phone number if set,
    #: otherwise defaults to call group name.
    last_name: Optional[str]
    #: Time zone for the call queue.
    time_zone: Optional[str]
    #: The alternate numbers feature allows you to assign multiple phone numbers or extensions to a call queue. Each
    #: number will reach the same greeting and each menu will function identically to the main number. The alternate
    #: numbers option enables you to have up to ten (10) phone numbers ring into the call queue.
    alternate_number_settings: Optional[AlternateNumberSettings]
    #: Policy controlling how calls are routed to agents.
    call_policies: Optional[CallPolicies]
    queue_settings: Optional[QueueSettings]
    allow_call_waiting_for_agents_enabled: Optional[bool]
    agents: List[Agent] = Field(default_factory=list)

    @staticmethod
    def create(name: str,
               agents: List[Agent],
               queue_size: int = None,
               enabled: bool = None,
               language_code: str = None,
               first_name: str = None,
               last_name: str = None,
               time_zone: str = None,
               phone_number: str = None,
               extension: str = None,
               call_policies: CallPolicies = None,
               queue_settings: QueueSettings = None,
               allow_call_waiting_for_agents_enabled: bool = None) -> 'CallQueueDetail':
        """
        Get an instance which can be uses for a create() call. Allows simplified creation of default queue settings
        based on queue_size
        :param name:
        :param agents:
        :param queue_size:
        :param enabled:
        :param language_code:
        :param first_name:
        :param last_name:
        :param time_zone:
        :param phone_number:
        :param extension:
        :param call_policies:
        :param queue_settings:
        :param allow_call_waiting_for_agents_enabled:
        :return:
        """
        if not (queue_size or queue_settings):
            raise ValueError('One of queue_size and queue_settings has to be given')
        if queue_size and queue_settings:
            raise ValueError('Only one of queue_size and queue_settings can be given')
        if not (phone_number or extension):
            raise ValueError('One of phone_number and extension has to be given')
        if queue_size:
            queue_settings = QueueSettings(queue_size=queue_size)
        params = {k: v for k, v in locals().items()
                  if v is not None and k != 'queue_size'}
        return CallQueueDetail(**params)


class CallQueueApi:
    """
    Call Queue APÍ

    :ivar forwarding: forwarding API :class:`wxc_sdk.telephony.forwarding.ForwardingApi`
    :ivar announcement: announcement API :class:`announcement.AnnouncementApi`

    """

    def __init__(self, session: RestSession):
        self._session = session
        self.forwarding = ForwardingApi(session=session, feature_selector=FeatureSelector.queues)
        self.announcement = AnnouncementApi(session=session)

    def _endpoint(self, location_id: str = None, queue_id: str = None):
        """
        Helper to get URL for API endpoints

        :meta private:
        :param location_id:
        :param queue_id:
        :return:
        """
        if location_id is None:
            return self._session.ep('telephony/config/queues')
        else:
            ep = self._session.ep(f'telephony/config/locations/{location_id}/queues')
            if queue_id:
                ep = f'{ep}/{queue_id}'
            return ep

    def list(self, location_id: str = None, name: str = None, org_id: str = None) -> Generator[CallQueue]:
        """
        Read the List of Call Queues
        :param location_id: Only return call queues with matching location ID.
        :param name: Only return call queues with the matching name.
        :param org_id: List call queues for this organization
        :return:
        """
        params = {to_camel(k): v
                  for i, (k, v) in enumerate(locals().items())
                  if i and v is not None}
        url = self._endpoint()
        # noinspection PyTypeChecker
        return self._session.follow_pagination(url=url, model=CallQueue, params=params)

    def by_name(self, name: str, location_id: str = None, org_id: str = None) -> Optional[CallQueue]:
        """
        Get queue info by name
        :param location_id:
        :param name:
        :param org_id:
        :return:
        """
        return next((cq for cq in self.list(location_id=location_id, org_id=org_id, name=name)
                     if cq.name == name), None)

    def create(self, location_id: str, queue: CallQueueDetail, org_id: str = None) -> str:
        """
        Create a Call Queue
        :param location_id:
        :param queue:
        :param org_id:
        :return: queue id
        """
        params = org_id and {'orgId': org_id} or {}
        if not queue.call_policies:
            queue.call_policies = CallPolicies.simple()
        cq_data = queue.json()
        url = self._endpoint(location_id=location_id)
        data = self._session.rest_post(url, data=cq_data, params=params)
        return data['id']

    def delete_queue(self, location_id: str, queue_id: str, org_id: str = None):
        """
        Delete queue
        :param location_id:
        :param queue_id:
        :param org_id:
        :return:
        """
        url = self._endpoint(location_id=location_id, queue_id=queue_id)
        params = org_id and {'orgId': org_id} or None
        self._session.delete(url=url, params=params)

    def details(self, location_id: str, queue_id: str, org_id: str = None) -> CallQueueDetail:
        """
        Read Settings for a Call Queue
        :param location_id:
        :param queue_id:
        :param org_id:
        :return:
        """
        url = self._endpoint(location_id=location_id, queue_id=queue_id)
        params = {'orgId': org_id} if org_id is not None else {}
        data = self._session.rest_get(url, params=params)
        result = CallQueueDetail.parse_obj(data)
        # noinspection PyTypeChecker
        return result

    def update_callqueue(self, location_id: str, queue_id: str, queue: CallQueueDetail, org_id: str = None):
        """
        Configure a Call Queue

        :param location_id:
        :param queue_id:
        :param queue:
        :param org_id:
        :return: queue id
        """
        params = org_id and {'orgId': org_id} or None
        cq_data = queue.json(exclude={'id', 'language', 'location_name', 'location_id', 'toll_free_number'})

        url = self._endpoint(location_id=location_id, queue_id=queue_id)
        self._session.rest_put(url=url, data=cq_data, params=params)



from enum import Enum
from typing import Optional

from pydantic import Field

from ..base import ApiModel

__all__ = ['UserType', 'UserBase']


class UserType(str, Enum):
    people = 'PEOPLE'
    place = 'PLACE'


class UserBase(ApiModel):
    first_name: Optional[str]
    last_name: Optional[str]
    user_type: UserType = Field(alias='type', default=UserType.people)



import base64
import sys
from typing import Optional

from pydantic import BaseModel, ValidationError


def webex_id_to_uuid(webex_id: Optional[str]) -> Optional[str]:
    """
    Convert a webex id as used by the public APIs to a UUID

    :param webex_id: base 64 encoded id as used by public APIs
    :type webex_id: str
    :return: ID in uuid format
    """
    return webex_id and base64.b64decode(f'{webex_id}==').decode().split('/')[-1]


def to_camel(s: str) -> str:
    """
    Convert snake case variable name to camel case
    log_id -> logId

    :param s: snake case variable name
    :return: Camel case name
    """
    return ''.join(w.title() if i else w for i, w in enumerate(s.split('_')))


class ApiModel(BaseModel):
    """
    Base for all models used by the APIs
    """

    class Config:
        alias_generator = to_camel  # alias is camelcase version of attribute name
        allow_population_by_field_name = True
        #: set to 'forbid' if run in unittest to catch schema issues during tests
        #: else set to 'allow'
        extra = 'allow' if 'unittest' not in sys.modules.keys() else 'forbid'

    def json(self, *args, exclude_none=True, by_alias=True, **kwargs) -> str:
        return super().json(*args, exclude_none=exclude_none, by_alias=by_alias, **kwargs)

    @classmethod
    def parse_obj(cls, obj):
        try:
            r = super().parse_obj(obj)
        except ValidationError as e:
            raise e
        return r
